#pragma once

#include "config.h"
#include "dataStructure.h"
#include "octreeLoader.h"
#include "renderEngine.h"
#include "master.h"
#include <mpi.h>

struct DepthBufferExchangeDescriptor
{
	int testerCount;
	int myIndex;
	int testers[TesterCount];
	void display();
};		


struct BinarySwapDescriptor
{
	int myTesterIndex;
	int myLeaderIndex;
	int myGroupSize;
	int myBuddyIndex;
	int myTileIndex;
	int myTileCount;
	int mySendIndex;
	int myRecvIndex;
	unsigned int mask;
	
	//bool isLow;
	void init(int testerIndex);
	bool next();	
	void display();
};

struct TesterContext
{
	static char* windowTitle;
	unsigned int myTesterIndex;
	RenderEngine renderEngine;
	OctreeLoader octreeLoader;
	MasterContext masterContext;
	int depthBufferCount;
	MPI_Request** visibleTableRenderRequests;
	MPI_Request* visibleTableDisplayerRequests;
	MPI_Request* snapShotSendingRequests;
	MPI_Request* snapShotRecvingRequests;
	MPI_Request* depthBufferExchangeRequests;
	MPI_Request* visibleTableExchangeRequests;
	//MPI_Request* naiveDepthBufferExchangeRequests;
	//MPI_Request* naiveVisibleTableExchangeRequests;
	
	//MPI_Request* naiveVisibleTableDisplayerRequests;

	MPI_Comm testerComm;
	MPI_Group testerGroup;
		
	int* snapShotCandidate;
	float* snapShotBuffer;
	float** snapShots;
	float** depthBuffer;
	ubyte** visibleTable;
	ubyte* colorBuffer;
	ubyte* renderIndexHashTable;
	float* myDepthBuffer;
	float* hisDepthBuffer;
	ubyte* myVisibleTable;
	ubyte* hisVisibleTable;
	///////////////////////////////
	//add an compression buffer for visible table
	ubyte* compressedVisibleTable;
	BinarySwapDescriptor* binarySwapDescriptor;
	DepthBufferExchangeDescriptor* exchangeDescriptor;	
	//return the length of compressed run-length
	//int compressVisibleTable(ubyte*source, int tileIndex);
	//void decompressVisibleTable(ubyte*target, int tileIndex, int count);
	/////////////////////////////////
	void init();

	void keyboardCallback(unsigned char key, int x, int y);

	void displayCallback();

	void readColorBuffer();
	void readDepthBuffer();
	void createRenderIndexHashTable();

	void processDepthBuffer();
	void exchangeDepthBufferSnapShot();
	void exchangeDepthBuffer();

	void doBinaryExchange();
	void sendVisiblePixelInfo();
	void testVisiblePixelInfoSending();
	void doExchangeDepthBuffer(int tileCount);
	void startExchangeDepthBuffer(int tileIndex);
	void finishExchangeDepthBuffer(int tileIndex);
	void enterNextRound(int tileIndex);
	void sendExchangeDepthBuffer(int tileIndex);
	void recvExchangeDepthBuffer(int tileIndex);
};

static TesterContext testerContext;

