#pragma once
#include "syscall.h"
#include "config.h"
#include "octree.h"
#include <mpi.h>
#include <set>
#include <stdio.h>

using namespace std;

struct MyPoint
{
	int x;
	int y;
	void operator=(const MyPoint& other)
	{
		x=other.x;
		y=other.y;
	}
	bool operator==(const MyPoint& other) const
	{
		return x==other.x&&y==other.y;
	}
};

struct PointComp
{
	bool operator()(const MyPoint& first, const MyPoint& second) const
	{
		return first.x<=second.x&&first.y<=second.y;
	}
};
		
struct Rect
{
	MyPoint topLeft;
	MyPoint bottomRight;
	bool operator==(const Rect& other) const
	{
		return topLeft==other.topLeft&& bottomRight==other.bottomRight;
	}
	void operator =(const Rect& other)
	{
		topLeft=other.topLeft;
		bottomRight=other.bottomRight;
	}
	bool isInside(const Rect& other) const
	{
		return topLeft.x>=other.topLeft.x&&topLeft.y>=other.topLeft.y
			&&bottomRight.x<=other.bottomRight.x&&bottomRight.y<=other.bottomRight.y;
	}
};

struct Region
{
	Rect rect;
	float maxz;
	float minz;
};

struct RegionComp
{
	bool operator() (const Region& first, const Region& second) const
	{
		return first.rect.bottomRight.x<=second.rect.topLeft.x||first.rect.bottomRight.y<=second.rect.topLeft.y;
	}
};		

typedef set<Region, RegionComp> RegionSet;
typedef float MatrixType[16];

struct DepthBufferPacket
{
	Region region;
	float depthBuffer[DepthBufferSize];
};	

struct Occluder
{
	FILE* log;
	DepthBufferPacket depthBufferPacket[TesterCount];
	int myTesterRank;
	Region myRegion;
	MatrixType transformer;
	float depthBuffer[DepthBufferSize];
	float buffer[DepthBufferSize];
	Region regionBuf[TesterCount];
	RegionSet regionSet;
	MPI_Comm occluderComm;
	MPI_Group occluderGroup;
	Occluder();
	void init(int rank);
	void uninit();
	void transformVertex(MatrixType transformer, FVector vertex, FVector result);
	bool exchangeRegion();
	void calcTransform();
	void readRegion(Rect& rect);
	void analysisRegion();
	void updateRegion();
	void insertRegion();
	bool intersectRegion(Region*ptr, RegionSet::iterator it);
	void matrixMultiply(MatrixType left, MatrixType right, MatrixType result);
	//bool intersectRegion(const Region& src, Region& dest, Region& interRgn);
	bool calcRegion(FVector center, float width, Rect& rect, float& depth);
	bool calcRegion(PlyVertex* vPtr, UIVector* fPtr, int faceNumber, Rect& rect, float& depth);
	void doCalcRegion(VVector src, Rect& rect, float& depth);
	void initCalcRegion(Rect& rect, float& depth);
	bool testCalcRegion(Rect& rect);
	bool testRegion(Rect& rect, float& minDepth);
	void sendEndExchange();
	void doInsertRegion(Region& rgn);
	void outputRegionSet();
	bool checkRegion(Rect rect, float depth);
	void mergeDepthBuffer();
	void printDepthBuffer();
	void printDepthBuffer(Rect rect);
	void checkDepthBuffer();	
	int doOccluderTest(PlyVertex* vPtr, UIVector* fPtr, int faceNumber);
		
};
	

