#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <errno.h>
#include <dirent.h>

char* HomePath="/nfs/home/q/qingz_hu";
const int ProfileItemCount=9;
const int MaxNodeNumber=256;

const unsigned int MaxTesterChoice=4;
const unsigned int MaxRenderChoice=28;
const unsigned int MaxScriptNumber=MaxTesterChoice*MaxRenderChoice;
const unsigned int MinTesterChoice=2;
const unsigned int MinRenderChoice=1;

const unsigned int TermNumber=6;
const unsigned int TesterChoice[MaxTesterChoice]=
{
	2,4,8,16
};

enum TermEnum
{
	MI=0, RS=1, RC=2, TS=3, TC=4, DI=5
};

const char* TermName[TermNumber]=
{
	"MasterIndex",   //MI
	"RenderHostStartIndex",  //RS
	"RenderCount",           //RC
	"TesterHostStartIndex",
	"TesterCount",
	"DisplayerIndex"
};


const static char* NodeName[4]=
{
        "Master Node",
        "Render Node",
        "Tester Node",
        "Displayer Node"
};

static char* ProfileItemName[ProfileItemCount]=
{
        "computation of Master",
        "communication from Master to Tester, Render and Displayer",
        "computation of Tester",
        "communication within Tester",
        "communication from Tester to Render, Displayer",
        "computation of Render",         
        "communication from Render to Displayer",
        "computation of Displayer",
        "whole system"  
};



float total[4][MaxNodeNumber][ProfileItemCount]={0};
char nodeName[4][MaxNodeNumber][10];
int nodeCount[4]={0};

enum ProfileItem
{
        CM=0, CT=2, CR=5, CD=7, BMTRD=1, BTRD=4, BRD=6, BT=3,  ALL=8
};



enum NodeStatus                 
{
        MASTER, RENDER, TESTER, DISPLAYER
};

void printResult(FILE* output)
{
	float result[ProfileItemCount];
	int i, j, k;
	//memset(result, 0, sizeof(float)*ProfileItemCount);
	for (i=0; i<ProfileItemCount; i++)
	{
		result[i]=0.0;
	}
	for (i=0; i<4; i++)
	{
		total[i][MaxNodeNumber-1][ALL]/=nodeCount[i];
		for (j=0; j<nodeCount[i]; j++)
		{
			for (k=0; k<ProfileItemCount; k++)
			{
				if (total[i][j][k]>0.0)
				{
					if (result[k]==0)
					{
						result[k]=total[i][j][k];
					}
					else
					{
						if (result[k]>total[i][j][k])
						{
							result[k]=total[i][j][k];
							//printf("%s=%.2f\n", ProfileItemName[k], total[i][j][k]);
						}
					}
				}					
				//printf("%.0f  ", total[i][j][k]);
			}
		}
	}
	//printf("fps=%.2f\n", 500*1000000.0/total[0][MaxNodeNumber-1][ALL]);
	//printf("%.2f  %.2f  %.2f  %.2f\n", result[BMTRD], result[BT], result[BTRD], result[BRD]);	
	fprintf(output, "%.2f  %.2f  %.2f  %.2f %.2f\n", result[BMTRD], result[BT], result[BTRD], result[BRD], total[0][0][ALL]);	
}
		


void retrieveTerm(char* buf, char* term, char* value)
{
	char* startPos, *endPos;

	startPos=strchr(buf, '[');
	endPos=strchr(buf, ']');
	if (startPos==NULL || endPos==NULL)
	{
		perror("retrieve term error:");
		exit(2);
	}
	
	strncpy(term, startPos+1, endPos-startPos-1);
	term[endPos-startPos-1]='\0';
	//printf("term=%s\n", term);

	startPos=strchr(endPos, '[');
	//printf("startPos=%s\n", startPos);
	endPos=strchr(startPos, ']');
	//printf("endPos=%s\n", endPos);

	if (startPos==NULL||endPos==NULL)
	{
		perror("retrieve term error:");
		exit(2);
	}

	strncpy(value, startPos+1, endPos-startPos-1);
	value[endPos-startPos-1]='\0';
}

NodeStatus whichNode(char* buf)
{
	char term[256];
	char value[256];
	retrieveTerm(buf, term, value);
	//printf("whichNode:buf=%s,term=%s,value=%s\n", buf, term, value);
	for (int i=0; i<4; i++)
	{
		if (strcmp(term, NodeName[i])==0)
		{
			strcpy(nodeName[i][nodeCount[i]], value);
			return (NodeStatus)(i);
		}
	}
	printf("error! node status unknown\n");
	exit(1);
}

ProfileItem whichItem(char* term)
{
	int i;
	//printf("term=%s\n", term);
	for (i=0; i<ProfileItemCount; i++)
	{
		if (strcmp(term, ProfileItemName[i])==0)
		{
			break;
		}
	}
	if (i==ProfileItemCount)
	{
		perror("cannot find item:");
		exit(3);
	}
	return (ProfileItem)(i);
}
	
				
void processData(NodeStatus nodeStatus, char* buf)
{
	char term[256];
	char value[256];
	ProfileItem item;
	int i;
	retrieveTerm(buf, term, value);
	item=whichItem(term);
	sscanf(value, "%d", &i);
	total[nodeStatus][nodeCount[nodeStatus]][item]=i;
	total[nodeStatus][MaxNodeNumber-1][item]+=i;
}
		 	
	
void processFile(char* fileName)
{
	FILE* file;
	int pos, i;
	char buf[256];
	NodeStatus nodeStatus;
	if ((file=fopen(fileName, "r"))==NULL)
	{
		printf("cannot open file %s\n", fileName);
		exit(1);
	}
	//determine node status
	if (fgets(buf, 256, file)==NULL)
	{
		perror("read error:");
		
	}
	//printf("get first line %s\n", buf);
	
	nodeStatus=whichNode(buf);
	while (fgets(buf, 256, file))
	{
		processData(nodeStatus, buf);
	}	
	nodeCount[nodeStatus]++;
	fclose(file);
}			


void forAllFile(char* dirName)
{
        struct stat buf;
        char curFileName[256];
        int nameLen;
        int startPos;

        char* src, *tgt;
        DIR* dirPtr;
        struct dirent* direntPtr;
        //printf("begin searching %s...\n", dirName);

        if ((dirPtr=opendir(dirName))==NULL)
        {
                //perror("read dir");
                //exit(1234);
                //printf("unable open %s\n", dirName);
                return;
        }
        nameLen=strlen(dirName);
        strcpy(curFileName, dirName);
        curFileName[nameLen]='/';

	memset(nodeCount, 0, 4*sizeof(int));

        while ((direntPtr=readdir(dirPtr))!=NULL)
        {
		startPos=strlen(direntPtr->d_name)-4;
                if (direntPtr->d_name[0]=='n'&&strcmp(direntPtr->d_name+startPos, ".txt")==0)
                {
                        strcpy(curFileName+nameLen+1, direntPtr->d_name);
			//printf("find file %s\n", curFileName);
			processFile(curFileName);		
                }
        }
        closedir(dirPtr);
}

const float DataSize[5]=
{
	10.8, 20.3, 32.0, 42.0, 52.0
};

//bool once=true;
void forAllDir(char* path)
{
	char dirName[256];
	unsigned int tester, render;
	FILE* output;
	if ((output=fopen("./output.txt", "w"))==NULL)
	{
		printf("cannot create output file\n");
		exit(14);
	}
	for (int expand=0; expand<5; expand++)
	{
		for (int t=0; t<MaxTesterChoice; t++)
		{
			for (int r=0; r<MaxRenderChoice; r++)
			{
				tester=TesterChoice[t];
				render=MinRenderChoice+r;
				if (tester+render>30)
				{
					continue;
				}
				fprintf(output, "%.2f %u %u %u ", DataSize[expand], tester, render, tester+render+2);
				sprintf(dirName, "%s/expand%u/T%uR%u", path, expand+1, tester, render);
				forAllFile(dirName);
				printResult(output);

			}
		}
	}
	fclose(output);
}			    
				

				


int main(int argc, char** argv)
{
	if (argc!=2)
	{
		printf("usage: commSumTool.exe dir\n");
		return 0;
	}
	else
	{
		printf("begin process dir %s\n", argv[1]);
		forAllDir(argv[1]);
	}
	return 0;
}

