#include "config.h"
#include "octree.h"
#include "octreeLoader.h"
#include "viewCollector.h"
#include "renderEngine.h"

#include <vector>
#include <cstdio>
#include <string>
#include <GL/glut.h>
#include <cmath>

#ifndef LINUX_VERSION
#include "windows.h"
#include "except.h"
#pragma comment(lib, "glut32.lib")
int indexBuffer[MaxPossibleNodeCollectable];
#endif


using namespace std;

extern void myTestData();
//extern int convert(char* fileName);

void displayCallbackProc();

void keyboardCallbackProc(unsigned char key, int x, int y);

void init();

unsigned long frameCounter=0;
unsigned long lastFrameNo=0;
unsigned long lastTick=0;
unsigned long currentTick=0;


ViewCollector viewCollector;
OctreeLoader octreeLoader;
RenderEngine renderEngine;

FILE* dumpPtr=NULL;

void dumpAll()
{
	if (dumpPtr!=NULL)
	{
		fprintf(dumpPtr, "\n****************begin dump the info*************\n");
		octreeLoader.root->infoStruct.display(dumpPtr);
		fprintf(dumpPtr, "\n****************end dump the info*************\n");
	}
	fclose(dumpPtr);
}

int main(int argc, char** argv)
{
	int expandTimes;
	float width;
	atexit(dumpAll);

	dumpPtr=fopen("error.txt", "a+");
	if (dumpPtr==NULL)
	{
		printf("cannot create error output file\n");
		exit(1);
	}

	if (argc==1)
	{
		glutInit(&argc, argv);
		glutInitDisplayMode(GLUT_DOUBLE | GLUT_RGBA | GLUT_DEPTH | GLUT_ALPHA);
		glutInitWindowSize(ScreenSize, ScreenSize);
		glutInitWindowPosition(5, 5);
		glutCreateWindow("render tool");
		init();

		glutDisplayFunc(displayCallbackProc);
		glutKeyboardFunc(keyboardCallbackProc);
		glutMainLoop();
	}
	DWORD timeCount;
	timeCount=GenericGetTime();
	if (argc==2)
	{
		if (strcmp(argv[1], "fake")==0)
		{
			printf("\n*************begin create fake color file*****************\n");
			OctreeLoader::paintPseudoColor();			
		}
		else
		{
			if (strcmp(argv[1], "info")==0)
			{
				printf("\n**************now print out info****************\n");
				OctreeLoader::readInfo();
			}
			else
			{		
				if (strcmp(argv[1], "shift")==0)
				{
					printf("\n*************begin shift-expand by 8*****************\n");
					OctreeLoader::expandBy8();
				}
				else
				{
					if (strcmp(argv[1], "user")==0)
					{
						printf("\n*************begin user input *****************\n");
						myTestData();
					}
					else
					{
						printf("\n*************now add directory %s***************\n", argv[1]);
						mergeData(argv[1]);
					}
				}
			}
		}
	}
	else
	{
		if (argc==3)
		{
			if (strcmp(argv[1], "convert")==0)
			{
				mergeSingleFile(argv[2]);
			}
			else
			{
				sscanf(argv[2], "%d", &expandTimes); 

				printf("\n**********add directory %s and expand %s times*********\n", argv[1], argv[2]);
				mergeData(argv[1], expandTimes);			
			}
		}
		else
		{
			if (argc==4)
			{
				if (strcmp(argv[1], "shift")==0)
				{
					sscanf(argv[3], "%f", &width);
					printf("\n*******add directory %s and expand by %s *****\n", argv[1], argv[3]);
					mergeData(argv[2], width);
				}
			}
			else
			{
				if (argc==5)
				{
					if (strcmp(argv[1], "shift")==0)
					{
						sscanf(argv[3], "%f", &width);
						sscanf(argv[4], "%d", &expandTimes);
						printf("dir '%s' will be shifted by %f with %d times\n", argv[2], width, expandTimes); 
						mergeData(argv[2], width, expandTimes);
					}
				}
				else
				{
					printf("usage:octreetool.exe dirname expandtimes\n");
				}
			}
		}
	}

	timeCount=GenericGetTime()- timeCount;
	printf("file process takes time %d milli seconds\n", timeCount);
	return 0;
}

//ubyte* colorBuffer;
void init()
{
	octreeLoader.initialize(true);

	renderEngine.init(&octreeLoader);
	//printf("now renderEngine.movement:\n");
	//renderEngine.movement.display();
	renderEngine.lighting.lightEnabled=false;

	viewCollector.initialize(octreeLoader.root);
	//renderEngine.movement.x+=octreeLoader.root->width/2.0;
	//renderEngine.movement.y+=octreeLoader.root->width/2.0;
	//renderEngine.updatePosition();
	//colorBuffer=new ubyte[ColorBufferByteSize];
	//CHECK_NEW(colorBuffer);
}

set<int> myColor;

/*
void debugProgram()
{
	int result;
	int* intPtr;
	bool findIt;
	glReadBuffer(GL_FRONT);
	glReadPixels(0, 0, ScreenSize, ScreenSize,  GL_RGBA, GL_UNSIGNED_BYTE, colorBuffer);
	
	intPtr=(int*)(viewCollector.octreeNodeVector.begin());
	for (int i=0; i<ScreenSize; i++)
	{
		for (int j=0; j<ScreenSize; j++)
		{
			result=0;
			memcpy(&result, colorBuffer+4*(i*ScreenSize+j), 3);
			findIt=false;
			for (int k=0; k<viewCollector.octreeNodeVector.size(); k++)
			{
				if (result==intPtr[k])
				{
					findIt=true;
					break;
				}
			}
			if (!findIt)
			{
				if (myColor.find(result)==myColor.end())
				{
					printf("problem color %d not found\n", result);
					myColor.insert(result);				
				}
				//exit(23);
			}
			else
			{
				printf("find color %d\n", result);
			}
		}
	}
}

*/


void displayCallbackProc()
{
	//void* temp;


	glClearColor(1.0, 1.0, 1.0, 1.0);
	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

	
	//viewCollector.startCollecting(renderEngine.drawingContext.center);
	viewCollector.startCollecting(renderEngine.drawingContext.center);
	//this is another g++ nusance, it doesn't allow me to do type cast
	//temp=(void*)(viewCollector.octreeNodeVector.begin());


	//octreeLoader.addCandidates(viewCollector.octreeNodeVector.size(), (int*)(temp));
	//octreeLoader.addCandidates(viewCollector.octreeNodeVector.size(), (int*)(viewCollector.octreeNodeVector.begin()));
	
	for (int i=0; i<viewCollector.octreeNodeVector.size(); i++)
	{
		octreeLoader.addCandidate(viewCollector.octreeNodeVector[i]);
		//indexBuffer[i]=viewCollector.octreeNodeVector[i];
	}
	
	//octreeLoader.addCandidates(viewCollector.octreeNodeVector.size(), indexBuffer);


	octreeLoader.swapBuffer();
	glutSwapBuffers();

        currentTick=GenericGetTime();
        if (frameCounter!=0)
        {
                printf("frame#%u:fps=%f\n", frameCounter, 1000.0/(float)(currentTick-lastTick));
        }
        lastTick=currentTick;
        frameCounter++;
#ifdef USING_FORCE_REFRESH
	glutPostRedisplay();
#endif


}

void keyboardCallbackProc(unsigned char key, int x, int y)
{
	switch (key)
	{
	case 'h':
	case 'H':
		viewCollector.hierRenderMode=!viewCollector.hierRenderMode;
		printf("render mode %s\n", (viewCollector.hierRenderMode)?"hierachical":"flooding");
		break;
	case 'f':
	case 'F':
		viewCollector.fullRenderMode=!viewCollector.fullRenderMode;
		printf("full render mode is %s\n", viewCollector.fullRenderMode?"yes":"no");
		break;

	case 'm':
	case 'M':
		renderEngine.movement.display();
		printf("nodeset size=%d\n", octreeLoader.nodeSet[octreeLoader.currentIndex].size());
		printf("movement offset=%f\n", renderEngine.movementOffset);
		printf("vertexLoaded=%d, faceLoaded=%d\n", octreeLoader.loadedVertexCounter, 
			octreeLoader.loadedFaceCounter);
		printf("vertexRendered=%d, faceRendered=%d\n", viewCollector.vertexCounter, 
			viewCollector.faceCounter);
		printf("nearplane=%f, farplane=%f\n", renderEngine.drawingContext.zNear, renderEngine.drawingContext.zFar);
		octreeLoader.root->infoStruct.display();
		break;

	case '+':		
		if (octreeLoader.capacityFactor>=MaxCapacity)
		{
			octreeLoader.capacityFactor=MaxCapacity;
		}
		else
		{
			octreeLoader.capacityFactor*=2;
		}
		printf("capacityFactor=%d\n", octreeLoader.capacityFactor);
		break;
	case '-':
		octreeLoader.capacityFactor/=2;
		if (octreeLoader.capacityFactor==0)
		{
			octreeLoader.capacityFactor=1;
		}
		printf("capacityFactor=%d\n", octreeLoader.capacityFactor);
		break;
	case 'b':
	case 'B':
		//debugProgram();
		break;
	default:
		renderEngine.keyboardCallback(key, x, y);
		break;

	}
	glutPostRedisplay();	
}
