//#define TESTING 1

#include <mpi.h>
#include "config.h"
#include "dataStructure.h"
#include "renderEngine.h"
#include "tester.h"
#include "master.h"
#include "displayer.h"
#include "render.h"
#include <GL/glut.h>
#include <unistd.h>

#ifndef LINUX_VERSION
#include "except.h"
#pragma comment(lib, "glut.lib")
#endif

unsigned long frameCounter=0;
unsigned long lastFrameNo=0;
unsigned long lastTick=0;
unsigned long currentTick=0;

void displayCallbackProc();

void keyboardCallbackProc(unsigned char key, int x, int y);

void idleCallbackProc();

extern MPIContext mpiContext;

extern void getLastError();

char dumpFileName[32];

FILE* dumpPtr;

void dump()
{
	//PROFILE_DISPLAY();
	//PROFILE_PRINT();
	if (dumpPtr!=NULL)
	{
		fclose(dumpPtr);
	}
}


int main(int argc, char** argv)
{
	MPI_Init(&argc, &argv);

	mpiContext.init();
	gethostname(mpiContext.nodeName, 32);
	printf("host %s starts at rank %d\n", mpiContext.nodeName, mpiContext.myRank);	

	glutInit(&argc, argv);
	getLastError();
	glutInitDisplayMode(GLUT_DOUBLE | GLUT_RGBA | GLUT_DEPTH | GLUT_ALPHA);
	glutInitWindowSize(ScreenSize, ScreenSize);
#ifdef TESTING
	glutInitWindowPosition(5+(mpiContext.myRank%4)*ScreenSize, 5+(mpiContext.myRank%4)*ScreenSize);
#else
	glutInitWindowPosition(0, 0);
#endif
	strcpy(dumpFileName, mpiContext.nodeName);
	strcat(dumpFileName, ".txt");
	dumpPtr=fopen(dumpFileName, "w");
	if (dumpPtr==NULL)
	{
		printf("open dump file failed\n");
		exit(1);
	}
	mpiContext.profiler.filePtr=dumpPtr;
	if (atexit(dump)!=0)
	{
		printf("atexit fails\n");
		exit(10);
	}

	switch (mpiContext.myNodeStatus)
	{
	case MASTER:
		glutCreateWindow(masterContext.windowTitle);		
		masterContext.init();
		printf("master rank#%d finishes init\n", mpiContext.myRank);
		//glutIdleFunc(idleCallbackProc);
		break;
	case RENDER:
		glutCreateWindow(rendererContext.windowTitle);		
		rendererContext.init();
		printf("render rank#%d finishes init\n", mpiContext.myRank);
		break;		
	case TESTER:
		glutCreateWindow(testerContext.windowTitle);		
		testerContext.init();
		printf("tester rank#%d finishes init\n", mpiContext.myRank);
		break;
	case DISPLAYER:
		glutCreateWindow(displayerContext.windowTitle);		
		displayerContext.init();
		printf("displayer rank#%d finishes init\n", mpiContext.myRank);
		break;		
	}

	glutDisplayFunc(displayCallbackProc);
	glutKeyboardFunc(keyboardCallbackProc);
	glutPostRedisplay();
	glutMainLoop();
	return 0;
}

void displayCallbackProc()
{
	glClearColor(1.0, 1.0, 1.0, 1.0);
	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
	switch (mpiContext.myNodeStatus)
	{
	case MASTER:				
		masterContext.displayCallback();
		currentTick=GenericGetTime();
		if (frameCounter!=0)
		{
			printf("frame#%u:fps=%f\n", frameCounter, 1000.0/(float)(currentTick-lastTick));
		}
		lastTick=currentTick;
		break;
	case RENDER:			
		rendererContext.displayCallback();
		break;		
	case TESTER:
		testerContext.displayCallback();
		break;
	case DISPLAYER:
		displayerContext.displayCallback();
		break;
	}
	frameCounter++;

	glutSwapBuffers();
#ifdef USING_FORCE_REFRESH
	glutPostRedisplay();
#endif
}



void keyboardCallbackProc(unsigned char key, int x, int y)
{
	switch (mpiContext.myNodeStatus)
	{
	case MASTER:				
		masterContext.keyboardCallback(key, x, y);
		break;
	case RENDER:		
		break;		
	case TESTER:		
		break;
	}
	glutPostRedisplay();
}


void idleCallbackProc()
{
	switch(mpiContext.myNodeStatus)
	{
	case MASTER:
		mpiContext.renderContext->key='m';
		glutPostRedisplay();
		break;
	}
}
